/*
Copyright 2007, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universit�t Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.

For documentation and questions please use the web site
http://scatterweb.mi.fu-berlin.de and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2007
*/

#include <ScatterWeb.System.h>
#include <ScatterWeb.CC1020.h>
#include <string.h>

bool Process_radioHandler(struct netpacket_handler_args* args);

void Process_sensorHandler(void* sdata)
{

}


/**
 * @brief	Ping request
 *
 * Send a ping request to another node
 * Example: ping 24
 */
COMMAND( ping, 0, cmdargs )
{
  uint16_t node;

  node = String_parseUint16( cmdargs->args, NULL );
  if ( node > 0 ) {
  	Net_sendPing((netaddr_t)node);
  }
}

/**
 * @brief Echo Arguments
 *
 * Testcase 1: long command name
 * Testcase 2: argument splitting
 */
COMMAND( echoargs, CMDFLAG_SERIAL, cmdargs )
{
  uint16_t i;
  uint16_t count = Cmd_SplitArgs( cmdargs->args );printf("\ncount = %u\n",count);
  char* arg = (char*) cmdargs->args;

  for ( i = 1; i <= count; i++ ) {
    printf("%u: %*s\n", i, strlen(arg), arg);
    arg = ( char* )Cmd_GetNextArg( arg );
  }
  snprintf(cmdargs->response_args, cmdargs->response_bufsize, "%u args found", count );
}

// Software header.
APP(
    APPH_STATIC_APP,	// Application Type
    __APP_NAME,			// Application Name (8 chars)
    1, 0				// Aplication Version (2 bytes)
);


/*
   1. Open src/ScatterWeb.Process.c
   2. Write a new terminal commando
*/
      COMMAND(HelloMSB, CMDFLAG_SERIAL, cmdargs) {
           printf("Hello User\n");
      }
/*
   3. Implement a call counter
*/
      uint8_t counter = 0;
      COMMAND(HelloMSB1, CMDFLAG_SERIAL, cmdargs) {
           printf("Hello User - called %u times\r\n", counter++);
      }
/*
     Parse a parameter
*/
     uint8_t counter2 = 0;
     COMMAND(HelloMSB2, CMDFLAG_SERIAL, cmdargs) {
       uint16_t n = String_parseUint16(cmdargs->args, NULL);
       while(n--) {
         printf("Hello User - called %u times\n", ++counter2);
       }
     }
/*
     You can use every function in the stdlib ...
     ... unfortunately MSPGCC provides only a reduced stdlib

   1. Call a function once 5 s after system start
*/
      // void callMe() { /* some action */}
      void callMe(); // Forward-Declaration
      void Process_init() {
           System_registerCallback( C_RADIO, (fp_vp)Process_radioHandler );
           System_registerCallback( C_SENSOR, (fp_vp)Process_sensorHandler );
           Timers_add(10*1024, callMe, (void*) 0xFFFF);
      }
/*
   2. Call a function every 5 s
   3. Toggle the led in your periodic function
*/
      void callMe() {
           static uint8_t callMe_counter=0;
           printf( "callMe(): - called %u times\n", ++callMe_counter );
           Timers_add(50*1024, callMe, (void*) 0xFFFF);
           P5OUT ^= 0x80; // toggleLED();
      }


#define SOME_NUMBER 1


COMMAND(my_send, CMDFLAG_SERIAL, cmdargs) {
      netaddr_t dest = String_parseUint16(cmdargs->args, NULL);
/*   
   1. Create header
*/
      netpacket_send_args_t npsargs;
      Net_sendArgsInit(&npsargs);
/*
   2. Set necessary values
*/
      npsargs.netheader.to = dest; // SOME_NODE_ID;
      npsargs.netheader.type = USERDEFINED_PACKET+SOME_NUMBER;
      npsargs.netheader.flags = 0;
/*
   3. Add some data
*/
      uint8_t* data = "Hello";
//    uint8_t data [] = "Hello world!";
      npsargs.payload[0].buffer = (uint8_t*) data;
      npsargs.payload[0].size = strlen(data) + 1; 
      // = sizeof(data); korrekt bei "uint8_t data[] = "Hello world"!
      printf("payload[0].size=%d, payload[0].buffer=|%*s|\n" 
             ,npsargs.payload[0].size ,npsargs.payload[0].size
             ,npsargs.payload[0].buffer);
/*
   4. Send frame/packet
*/
      Net_send(&npsargs);
}

/*
   1. Goto Process radioHandler()
   2. Include case for your type

      switch ( args->netheader->type ) {
           case PING_REQUEST_PACKET:
                 Net_sendPong(args->netheader->from);
                 break;
           case PING_REPLY_PACKET:
                 LOG_LOW( "PONG from %u", args->netheader->from );
                 break;
           case USERDEFINED_PACKET+SOME_NUMBER:
                 // do something
                 break;
           default:
                 return false;
      }
*/

/*
   Dump Network Traffic
      -  Write a command that switches on/off the dumping of received packets.
      -  Print the raw header and data to the serial port.
      -  Decode the header fields in your output, e.g. To: 123.
      -  Enable promiscous mode; receive all packets and not only multicasts and unicast
         destined for your node
*/
bool dump=0;
COMMAND(toggle_dump, CMDFLAG_SERIAL, cmdargs) { 
      dump = ! dump; 
      printf("Dumping of received packets: %s", (dump?"ON":"OFF"));
}
bool Process_radioHandler(struct netpacket_handler_args* args)
{  // Ausser Acknowledgement-Paketen (abgefangen von Schicht 2) werden
  // an diese Funktion alle empfangenen Pakete weitergereicht :
  /*
    Notes: * The received packet data becomes invalid when this handler is completed.
  */
  // 12.107 -> 11, Typ 7 87 bytes
  // 12 -> [7,87,"Die Welt ist schrecklich          "] -> 11

  uint8_t *in_payload, n;
  in_payload = (uint8_t *) args->payload;
  n = args->payload_size;

  if(dump) { 
    printf("%u.%u -> %u, Typ %d %u bytes\n", args->netheader->from
           ,args->netheader->_num          , args->netheader->to
           ,args->netheader->type          , args->payload_size );
    putchar('[');
    while(n--) { 
      if(isprint(*(char*)in_payload)) putchar(*in_payload);
      else  printf("\\%u ", *in_payload);
      in_payload++;
    }
    putchar(']');
  }
  // Pass packet through command interface
  if ( Cmd_radioHandler( args ) ) { // packet handled by messaging module
    return true;
  }

  if ( args->cpaResult == ADRCOMP_OTHER ) {
    // intercept packets for other nodes here
  } else if ( args->cpaResult == ADRCOMP_BROADCAST ) {
    // process broadcast packets here
  } else {
    // process packets for this node here
    switch ( args->netheader->type ) {
      case PING_REQUEST_PACKET:
        // send back to receiver's address
        Net_sendPong(args->netheader->from);
        break;

      case PING_REPLY_PACKET:
        LOG_LOW( "PONG from %u", args->netheader->from );
        break;

      case USERDEFINED_PACKET+SOME_NUMBER:
       // do something
        printf("my_send_Paket(Typ %u)\n",USERDEFINED_PACKET+SOME_NUMBER);
        /*
        printf("my_send-Paket von %u an %u, %u Bytes: [%*s]\n",   
          args->netheader->from, args->netheader->to,
          args->payload_size, (args->payload_size - 3), args->payload    
         );  // Vorsicht: printf("%*s",len,string) arbeitet nicht wie spezifiziert
           // Bei nicht mit '\0' abgeschlossenem string werden mehr
             // als <len> Zeichen ausgegeben insbesondere auch solche, die
             // nicht mehr zum String gehoeren !!
        */
        break;
      default:
        return false;    // packet not handled (unknown)
    }
    return true;        // packet handled
  }
  return false;          // packet not handled (other destination)
}

//Auf dem Weg zum Traffic-Generator...
/*
'my_send1' soll das als 2. auf der seriellen Schnittstelle eingegebene String-Argument an 
die als erstes Argument angegebene Adresse verschicken.
*/

COMMAND(my_send1, CMDFLAG_SERIAL, cmdargs) {
      netaddr_t dest = String_parseUint16(cmdargs->args, NULL);
      /*   
         1. Create header
      */
      netpacket_send_args_t npsargs;
      Net_sendArgsInit(&npsargs);
      /*
         2. Set necessary values
      */
      npsargs.netheader.to = dest; // SOME_NODE_ID;
      npsargs.netheader.type = USERDEFINED_PACKET+SOME_NUMBER;
      npsargs.netheader.flags = 0;
      /*
         3. Add some data
      */

      /*  uint16_t i;
          uint16_t count = Cmd_SplitArgs( cmdargs->args );
          char* arg = (char*) cmdargs->args;
      */

      /*  for ( i = 0; i <= count; i++ ) {
            printf("%u: %*s\n", i, strlen(arg), arg);
            arg = ( char* )Cmd_GetNextArg( arg );
          } 
      */      
      
      //Uebertraegt alle auf der ser. Schnittstelle uebergebenen Argumente:
      //Pointer auf alle Argumente die auf der seriellen Schnittstelle eingegeben werden

      /*  uint8_t* data = cmdargs->args;
      //  uint8_t data [] = "Hello world!";
      npsargs.payload[0].buffer = (uint8_t*) data;
      npsargs.payload[0].size = strlen(data) + 1; 
      // = sizeof(data); korrekt bei "uint8_t data[] = "Hello world"!
      printf("payload[0].size=%d, payload[0].buffer=|%*s|\n" 
             ,npsargs.payload[0].size
             ,npsargs.payload[0].size
             ,npsargs.payload[0].buffer);
      */
      
      //Uebertraegt die Argumente nach der ZielknotenID:
      /*      uint8_t* data = cmdargs->args;
      //ueberspringe Leerzeichen am Anfang:
      while(*data==CHAR_SPC || *data==CHAR_TAB)
        *data++;
      //ueberspringe erste zahl(Adr. des Zielknotens):
      while(*data>='\x30' && *data<='\x39')
        *data++;
      npsargs.payload[0].buffer = (uint8_t*) data;
      npsargs.payload[0].size = strlen(data) + 1; 
      printf("payload[0].size=%d, payload[0].buffer=|%*s|\n" 
       ,npsargs.payload[0].size
       ,npsargs.payload[0].size
       ,npsargs.payload[0].buffer);
      */

      //
            uint8_t* data = cmdargs->args;
      //ueberspringe Leerzeichen am Anfang:
      while(*data==CHAR_SPC || *data==CHAR_TAB)
        *data++;
      //ueberspringe erste zahl(Adr. des Zielknotens):
      while(*data>='\x30' && *data<='\x39')
        *data++;
      npsargs.payload[0].buffer = (uint8_t*) data;
      npsargs.payload[0].size = strlen(data) + 1; 
      printf("payload[0].size=%d, payload[0].buffer=|%*s|\n" 
       ,npsargs.payload[0].size
       ,npsargs.payload[0].size
       ,npsargs.payload[0].buffer);
      
      /*
         4. Send frame/packet
      */
      Net_send(&npsargs);
}


/*
   3. Test the sending and receiving of data

   Traffic Generator
*/

#define MAX_TRAFFIC_GENERATORS 4
struct traffic_spec {
  uint8_t id,  // Kennzeichnung/Name eines einzelnen Traffic-Generators
               // Dies Feld ist im Moment UEBERFLUESSIG
    dest,      //++ Adresse des (Ziel-)Knotens an den der Traffic gesendet wird
    data_size; //++ the size of sent data per (Traffic-)packet
  uint16_t  interval,  // -  The interval shall be configurable as parameter.
    limit; // -  Limit the number of sent packets by another parameter.
} traffic_generators[MAX_TRAFFIC_GENERATORS]; 

uint8_t traffic_generators_initialized=0;
void traffic_send( struct traffic_spec* );

// - Write a command that can initiate and terminate 
//   the periodic sending of data.

  //Fallunterscheidung ob 2 oder mehr Argumente
  // terminate traffic_generator <x> : traffic_gen x 0  
  // initiate  traffic_generator <x> : 
  //   Syntax: traffic_gen <traffic_no> <dest> <interval> <limit> <data_size>
  //   Bsp.:  traffic_gen 3 12 300 10 50
  //          Erzeugen + Starten des Traffic-Generators 
  //          - mit Nummer 3 
  //          - zum Senden an Knoten 12 eines Pakets
  //          - alle 300 ms 
  //          - aber hoechstens 10 mal 
  //          - bestehend aus 50 Bytes
  //?? list Traffic-Generator's        : traffic_gen

COMMAND(traffic_gen,CMDFLAG_SERIAL,cmdargs){
  uint8_t n, generator_nr; // < MAX_TRAFFIC_GENERATORS
  // Kommando-Zeile  auswerten : 
  // Aufruf ohne Argumente -> list Traffic-Generator's
  if(! traffic_generators_initialized) // initialize traffic_generators[]
    for(n= traffic_generators_initialized = MAX_TRAFFIC_GENERATORS ;n--;){ 
      struct traffic_spec tmp = { 0,0,0,0,0 };
      traffic_generators[n] = tmp;       
      traffic_generators[n].id = n;
      traffic_generators[n].interval = traffic_generators[n].limit     = 
      traffic_generators[n].dest   = traffic_generators[n].data_size = 0;
    }
  
  char *pt=cmdargs->args,*ptt;
  if(*pt==0){//keine (Kommandozeilen-)argumente angegeben 
  //-> list Traffic-Generator's;
    for(generator_nr=0 ; generator_nr < MAX_TRAFFIC_GENERATORS ; generator_nr++)
      printf("%u: %u %u %u %u\n", generator_nr,
              traffic_generators[n].dest,
              traffic_generators[n].interval,
              traffic_generators[n].limit,
              traffic_generators[n].data_size );
    return;
  } 
  // pt=ptt;
  generator_nr = String_parseUint16(pt ,&ptt); 
  if(pt==ptt || generator_nr > MAX_TRAFFIC_GENERATORS){
     //printf("Fehler: es wurde keine zulaessige Zahl (aus 0,1,..%u) zur Benennung des Traffic-Generators angegeben !\n");
     printf("Fehler bei Generator-Id(0,..%u)\n", MAX_TRAFFIC_GENERATORS-1);
     return;
  }
  pt=ptt;
  // Wenn nur 2 Argumente bzw. 2.Argument (dest) 0 ist,
  // soll Generator entfernt werden und damit auch der i.A. existierende Timer
  if(0 == (traffic_generators[generator_nr].dest  = String_parseUint16(pt ,&ptt))
     && pt != ptt ) {
     Timers_remove(traffic_send, & traffic_generators[generator_nr]); 
     traffic_generators[n].limit = 0; // reicht 
     /* traffic_generators[n].interval = traffic_generators[n].limit   =
        traffic_generators[n].dest   = traffic_generators[n].data_size = 0; */
     // printf("Generator %u entfernt\n", generator_nr);
     return; 
  }
  if(pt==ptt){ //terminate traffic_generator <x> : traffic_gen x 
    traffic_generators[generator_nr].limit = 0;
    return;
  }
  pt=ptt;
  if(!(traffic_generators[generator_nr].interval = String_parseUint16(pt ,&ptt))){
    return; }
  pt=ptt;
  if(!(traffic_generators[generator_nr].limit = String_parseUint16(pt ,&ptt))){
    return; }
  pt=ptt;
  if(!(traffic_generators[generator_nr].data_size = String_parseUint16(pt ,&ptt))){
    return; }
  printf("  traffic_send(& traffic_generators[generator_nr]);\n");
  traffic_send(& traffic_generators[generator_nr]);
}

// Maximal-Zahl an von einander unabhaengigen Traffic-Generatoren/(Streams)
// Scatter-Web kann hoechstens 140 Byte grosse Pakete empfangen 
// (siehe Groesse der Empfangspuffer rxBuf1/rxBuf2)
// Die uebertragene Payload kann somit maximal aus 119 Bytes bestehen
//   119 = 140 - 8 - 9 - 2 - 2 
//         (8 Byte Header, 9 Byte Preamble, 2 Byte CRC, 2 Zeichen '0xFA')
// Auch wenn es moeglich ist groessere Pakete/Payloads zu VERSENDEN, 
// macht das keinen Sinn (da sie nicht vollstaending empfangen werden koennen):
#define MAX_PAYLOAD_SIZE 119
//void traffic_send( void *ts ){

/* -------------------- MY_SEND als Funktion oder Macro ------------- */
// (ts->dest, ts->data_size, data);
// #define MY_SEND(dest,data_size,pdata)
void MY_SEND(netaddr_t dest ,uint8_t data_size, char *pdata) {
      /* 1. Create header */
      netpacket_send_args_t npsargs;
      Net_sendArgsInit(&npsargs);
      /* 2. Set necessary values */
      npsargs.netheader.to = (dest); // SOME_NODE_ID;
      npsargs.netheader.type = USERDEFINED_PACKET+SOME_NUMBER;
      npsargs.netheader.flags = 0;
      /* 3. Add some data */
      npsargs.payload[0].buffer = (uint8_t*) (pdata);
      npsargs.payload[0].size = (data_size); 
      /* 4. Send frame/packet */
      Net_send(&npsargs);
}


/* #define MY_SEND(dest,data_size,pdata) \ */
#define NET_SEND3(dest,data_size,pdata) \
  do { netpacket_send_args_t npsargs; \
       Net_sendArgsInit(&npsargs); \
       npsargs.netheader.to = (dest); \
       npsargs.netheader.type = USERDEFINED_PACKET+SOME_NUMBER; \
       npsargs.netheader.flags = 0; \
       npsargs.payload[0].buffer = (uint8_t*) (pdata); \
       npsargs.payload[0].size = (data_size); \
       Net_send(&npsargs); \
  } while(0)


void traffic_send( struct traffic_spec *t ){
  //(struct traffic_spec *) 
  if(! t->limit--) return;
  // Sende (Paket aus) ts->data_size Bytes an Knoten ts->dest
  char data[MAX_PAYLOAD_SIZE];
  // Trage uebersichtliches Muster aus t->data_size Bytes in data[] ein :
  uint8_t i=0;
  for(i=0 ; i < t->data_size ; i++) data[i] = '0' + (i % 10);
  MY_SEND(t->dest, t->data_size, data);
  // Das naechste if ist eigentlich ueberfluessig , spart aber einmaliges
  // Eintragen/Behandeln eines unnoetigen Timers wenn t->limit == 0 :
  if(t->limit) // in ts->interval Millisekunden soll ein weiteres Paket 
    Timers_add(t->interval, traffic_send, (void*) t);
  /* Alternative Struktur, aber Timer-Eintrag vor MY_SEND() !!
     switch(limit) {
     case 0: return; break;
     case 1:  Timers_add(..);  // kein break;
     default: MY_SEND(ts->dest, ts->data_size, data);
     }
  */ 
}  

     // "ByteOrder" liefert : (Hoeherwertiges Byte ("AB") bei hoeherer Adresse !)
     // unsigned int ui=ABCD     ((uchar *)& us)[1..0]= AB CD 
     // 248e = & ((uint8_t *)& ui)[0] 
     // 248f = & ((uint8_t *)& ui)[1] 
     COMMAND(ByteOrder, CMDFLAG_SERIAL, cmdargs) {
            uint16_t n = String_parseUint16(cmdargs->args, NULL);
            uint16_t ui = 0xABCD;    
            unsigned char *puc = (unsigned char*) & ui;
            printf("sys/inttypes.h: typedef unsigned int  uint16_t\n");
            printf("sys/inttypes.h: typedef unsigned char uint8_t\n");

            printf("%u = sizeof(unsigned short),\
                    %u = sizeof(unsigned int)\n" ,sizeof(unsigned short)
                                       ,sizeof(unsigned int)   );
            printf("unsigned int ui=%hX     " ,ui); 
            printf("((uchar *)& ui)[1..0]= %hhX %hhX \n" , puc[1] , puc[0]);
            printf("%x = & ((uint8_t *)& ui)[0] \n", & puc[0]);
            printf("%x = & ((uint8_t *)& ui)[1] \n", & puc[1]);

     }

/*
dump/Payload-Ausgabe in radio_handler() verbessern
COMMAND traffic_gen:
 Loeschen/Ueberschreiben von Traffic-Generatoren aendern
  (Bei Ueberschreiben/Setzen muss nachgesehen werden,
   ob der alte Timer noch laeuft, wenn ja entfernen 
   (abwarten bis dieser auslaeuft kann zu lange dauern)
   Bei Loeschen muss evt. genauso verfahren werden.
  )
 Ausgeben der Traffic-Generatoren-Liste einbauen
 Parameter-Bearbeiten/Verarbeiten verbessern 
  (Wegfall von ptt fuer Argumente die nicht 0 sein duerfen)
  data_size + generator_nr duerfen 0 sein !?
  evt. Fehler-Ausgabe wenn Argument nicht korrekt ist 
Doxygen mindest. fuer Traffic-Generator-Usage
Beautify First Steps
Checkin  First Steps (ScatterWeb.Process.c)
*/
