/*
Copyright 2007, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
 
- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.

For documentation and questions please use the web site
http://scatterweb.mi.fu-berlin.de and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2007
*/

/**
 * @file			ScatterWeb.MD4.h
 * @ingroup	libMD4
 * @brief			MD4 library, Public interface
 * 
 * @author		Harald Sauff
 * @date			Aug 2007
 * @since 		1.2
 * @version	$Revision$
  *
 * <p>
 * Copyright (C) 1991-2, RSA Data Security, Inc. Created 1991. All
 * rights reserved.
 * <p>
 * License to copy and use this software is granted provided that it
 * is identified as the "RSA Data Security, Inc. MD4 Message-Digest
 * Algorithm" in all material mentioning or referencing this software
 * or this function.
 * <p>
 * License is also granted to make and use derivative works provided
 * that such works are identified as "derived from the RSA Data
 * Security, Inc. MD4 Message-Digest Algorithm" in all material
 * mentioning or referencing the derived work.
 * <p>
 * RSA Data Security, Inc. makes no representations concerning either
 * the merchantability of this software or the suitability of this
 * software for any particular purpose. It is provided "as is"
 * without express or implied warranty of any kind.
 * <p>
 * These notices must be retained in any copies of any part of this
 * documentation and/or software.
 * <p>
 * - source code taken from reference implementation in RFC 1320
 * - adapted by Harald Sauff,
 *   Technical University of Hamburg-Harburg (TUHH)
 *
 */

/**
 * @ingroup	libMD4
 * @{
 */

#ifndef __MD4_H__
#define __MD4_H__

#include <ScatterWeb.System.h>

/******************************************************************************
 * @name	 initialisation and usage
 * @{
 */

/**
 * @brief MD4 checksum calculation
 *
 * Generates a MD4 checksum
 *
 * @param input		pointer to input to generate a MD4 checksum of
 * @param length	length of input
 * @param output	output of the MD4 checksum
 * @return					Pointer of output.
 * @since                 	1.2
 * @version 				$Revision$
 */
uint8_t* MD4_get(uint8_t *input, uint16_t length, uint8_t *output);

/**
 * @brief Required dummy function
 */
void libMD4_Init() {}

#endif /*__MD4_H__*/
/** @} */
