/**
 * @file		libscatterweb.h
 * @ingroup		config, config_compile
 * @brief		Default compile-time configuration of library scatterweb
 * @version		$Revision: 241 $
 *
 * $Id: libscatterweb.h 241 2007-11-09 13:52:36Z baar@PCPOOL.MI.FU-BERLIN.DE $
 */

/**
 * @brief	Library Initialiser
 * @internal
 */
void libscatterweb_Init();

/**
 * @addtogroup	config_compile
 * @{
 */
#ifndef DEBUGLOG_LEVEL
	/**
	 * @brief Debuglog Output Level (determines which logmessages get compiled)
	 * @since 1.1
	 * 
	 * \li 0: None (quiet)
	 * \li 1: Low (includes assertions)
	 * \li 2: Medium
	 * \li 4: High
	 * \li 5: Verbose
	 * 
	 * @see ScatterWeb.Debuglog.h
	 */
 	#define DEBUGLOG_LEVEL			5
#endif

#ifndef SYSTEM_CMDLEVEL
	/**
	 * @brief Command Interface Level
	 * @since 1.1								
	 * 
	 * Determines which commands are included in the application.
	 * \li 0: None									
	 * \li 1: ScatterWeb Tools						
	 * \li 2: ScatterWeb Tools and manual management
	 * \li 4: Testing								
	 * \li 5: Debugging
	 * 
	 */
	#define SYSTEM_CMDLEVEL		5
#endif

#ifndef NETPACKET_SEND_BUFFER_COUNT
	/**
	 * @brief	Number of buffers in net send argument struct
	 * @see		netpacket_send_args
	 */
	#define NETPACKET_SEND_BUFFER_COUNT 2
#endif

#ifndef WITH_ASSERT
	/**
	 * @brief	Enable (compile) assertions
	 */
	#define WITH_ASSERT			1
#endif

/** @} */

